Struct UnionFind

pub struct UnionFind<Value> { /* private fields */ }
A basic implementation of a union-find datastructure.

Implementations

Source

impl<Value: NumericId> UnionFind<Value>

pub fn reset(&mut self)

Reset the union-find data-structure to the point where all Ids are their own parents.

pub fn reserve(&mut self, v: Value)

Reserve sufficient space for the given value v.

pub fn union(&mut self, a: Value, b: Value) -> (Value, Value)

Merge two equivalence classes.

pub fn find(&mut self, id: Value) -> Value

Find the representative of an equivalence class.

pub fn find_naive(&self, id: Value) -> Value

Find the representative of an equivalence class without using path compression.

The primary advantage of this method is that it allows the ability to answer find queries without holding a mutable reference to the union-find.