pub struct UnionFind<Value> { /* private fields */ }Expand description
A basic implementation of a union-find datastructure.
Implementations
Sourceimpl<Value: NumericId> UnionFind<Value>
impl<Value: NumericId> UnionFind<Value>
Sourcepub fn reset(&mut self)
pub fn reset(&mut self)
Reset the union-find data-structure to the point where all Ids are their own parents.
Sourcepub fn union(&mut self, a: Value, b: Value) -> (Value, Value)
pub fn union(&mut self, a: Value, b: Value) -> (Value, Value)
Merge two equivalence classes.
Sourcepub fn find_naive(&self, id: Value) -> Value
pub fn find_naive(&self, id: Value) -> Value
Find the representative of an equivalence class without using path compression.
The primary advantage of this method is that it allows the ability to answer find queries
without holding a mutable reference to the union-find.